<?php

namespace App\Console\Commands;

use App\Log as AppLog;
use App\Models\Goods;
use App\RedisModel;
use App\Service\GoodsListService;
use App\Support\EmailQueue;
use Exception;
use Illuminate\Console\Command;

class SyncGoodsList extends Command
{
    protected $signature = 'sync:goods-list';
    protected $description = '同步商品列表数据';
    
    private $startTime; 
    const MAX_EXECUTION_TIME = 600; // 10分钟超时时间（秒）
    const QUEUE_ALERT_THRESHOLD = 1000; // 队列长度预警阈值

    public function __construct()
    {
        parent::__construct();
        $this->startTime = time();
    }

    public function handle()
    {
        try {
            $this->info('开始同步商品列表...');
            
            // 检查队列长度
            $queueLength = RedisModel::lLen(Goods::QUEUE_KEY);
            if ($queueLength > self::QUEUE_ALERT_THRESHOLD) {
                EmailQueue::rPush('商品详情队列长度超过预警阈值', json_encode(['queue_length' => $queueLength, 'queue_key' => Goods::QUEUE_KEY], 256),
                    ['song.shen@kuxuan-inc.com'], '微店推客带货');
                $this->warn("队列积压严重，当前长度:".$queueLength);
                return;
            }

            // 开始同步之前先将数据状态重置
            Goods::resetStatus();
            
            $nextKey = '';
            $hasMore = true;
            $spuItemCondition = null;
            $category = null;
            $keyword = null;

            $planTypeList = [2];// 1:定向计划 2:公开计划
//            $planTypeList = [2, 1];// 1:定向计划 2:公开计划

            foreach ($planTypeList as $planType) {
                do {
                    // 检查超时
                    if ($this->shouldExit()) {
                        break;
                    }
                    
                    try {
                        $result = GoodsListService::processGoodsList($planType, $spuItemCondition, $category, $keyword, $nextKey);
                        
                        if (!$result['success']) {
                            $this->info('没有新的商品数据');
                            break;
                        } else {
                            $this->info('处理商品列表批次成功,next_key: '.$result['next_key']);
                        }
                        
                        // 更新分页标识
                        $nextKey = $result['next_key'];
                        
                    } catch (Exception $e) {
                        $errorParams = ['next_key' => $nextKey, 'plan_type' => $planType, 'file' => $e->getFile(), 'line' => $e->getLine(), 'error' => $e->getMessage(), 'trace' => $e->getTraceAsString()];
                        AppLog::logError("处理商品列表批次失败", $errorParams, 'sync_goods_list');
                        EmailQueue::rPush('处理商品列表批次失败', json_encode($errorParams, 256), ['song.shen@kuxuan-inc.com'], '微店推客带货');
                        continue;
                    }
                    
                } while ($nextKey !== '');
            }
            
        } catch (Exception $e) {
            $errorParams = ['file' => $e->getFile(), 'line' => $e->getLine(), 'error' => $e->getMessage(), 'trace' => $e->getTraceAsString()];
            $this->error('同步失败：' . $e->getMessage());
            AppLog::logError('商品列表同步失败：' . $e->getMessage(), $errorParams, 'sync_goods_list');
            EmailQueue::rPush('商品列表同步失败', json_encode($errorParams, 256), ['song.shen@kuxuan-inc.com'], '微店推客带货');
        }
    }

    private function shouldExit(): bool
    {
        if (time() - $this->startTime >= self::MAX_EXECUTION_TIME) {
            $this->warn('脚本已运行10分钟，自动退出');
            return true;
        }
        return false;
    }
} 