<?php

namespace App\Console\Commands;

use App\Log as AppLog;
use App\Models\Order;
use App\RedisModel;
use App\Service\OrderService;
use App\Support\EmailQueue;
use Exception;
use Illuminate\Console\Command;

class SyncOrderDetail extends Command
{
    protected $signature = 'sync:order-detail';
    protected $description = '同步佣金单详情数据';
    
    private $startTime;
    const MAX_EXECUTION_TIME = 600; // 10分钟超时

    private $orderService;

    public function __construct(OrderService $orderService)
    {
        parent::__construct();
        $this->startTime = time();
        $this->orderService = $orderService;
    }

    public function handle()
    {
        try {
            $this->info('开始同步佣金单详情...');
            
            while (!$this->shouldExit()) {
                $orderInfo = RedisModel::rPop(Order::ORDER_INFO_QUEUE);
                if (!$orderInfo) {
                   sleep(1);
                   continue;
                }

                $this->processOrder($orderInfo);
            }
            
        } catch (Exception $e) {
            $this->error('同步失败：' . $e->getMessage());
            $this->orderService->handleError('同步失败', $e);
        }
    }

    private function processOrder($orderInfo)
    {
        \DB::beginTransaction();
        try {
            $detail = $this->orderService->processOrderDetail(
                $orderInfo['order_id'],
                $orderInfo['sku_id']
            );
            if(empty($detail)) {
                return;
            }

            Order::saveOrderDetail($detail);

            \DB::commit();
            
        } catch (Exception $e) {
            \DB::rollBack();
            $errorParams = ['orderInfo' => $orderInfo, 'file' => $e->getFile(), 'line' => $e->getLine(), 'error' => $e->getMessage(), 'trace' => $e->getTraceAsString()];
            AppLog::logError('同步佣金单详情失败', $errorParams, 'sync_order_detail'); 
            EmailQueue::rPush('同步佣金单详情失败', json_encode($errorParams), ['song.shen@kuxuan-inc.com'], '微店推客带货');
        }
    }

    private function shouldExit(): bool
    {
        if (time() - $this->startTime >= self::MAX_EXECUTION_TIME) {
            $this->warn('脚本已运行10分钟，自动退出');
            return true;
        }
        return false;
    }
} 