<?php

namespace App\Console\Commands;

use App\Log as AppLog;
use App\Models\Order;
use App\Service\OrderService;
use App\Support\EmailQueue;
use Exception;
use Illuminate\Console\Command;

class SyncOrderList extends Command
{
    protected $signature = 'sync:order-list {--days=1}';
    protected $description = '同步佣金单列表数据';
    
    private $startTime;
    const MAX_EXECUTION_TIME = 600;
    const QUEUE_ALERT_THRESHOLD = 5000;

    private $orderService;

    public function __construct(OrderService $orderService)
    {
        parent::__construct();
        $this->startTime = time();
        $this->orderService = $orderService;
    }

    public function handle()
    {
        try {
            $this->info('开始同步佣金单列表...');
            $appId = config('wechat_api.config.app_id');
            
            // 检查队列长度
            $queueLength = $this->orderService->getQueueLength();
            if ($queueLength > self::QUEUE_ALERT_THRESHOLD) {
                AppLog::logError("佣金单队列长度超过预警阈值: {$queueLength}");
                EmailQueue::rPush('佣金单队列长度超过预警阈值', json_encode(['queueLength' => $queueLength, 'queueKey' => Order::ORDER_INFO_QUEUE]), ['song.shen@kuxuan-inc.com'], '微店推客带货');  
                $this->warn("队列积压严重，当前长度: {$queueLength}");
                return;
            }
            
            // 计算时间范围
            $days = $this->option('days');
            $endTime = time() ;
            $startTime = time() - $days * 24 * 60 * 60;
            
            $nextKey = '';
            
            do {
                if ($this->shouldExit()) {
                    break;
                }
                
                try {
                    $result = $this->orderService->processOrderList(
                        $appId,
                        $startTime,
                        $endTime,
                        $nextKey
                    );
                    
                    if (!$result['success'] || !$result['has_more']) {
                        $this->info('没有新的佣金单数据');
                        break;  
                    }
                    
                    $nextKey = $result['next_key'];
                    
                } catch (Exception $e) {
                    $this->error($e->getMessage());
                    continue;
                }
                
            } while ($nextKey !== '' && $result['has_more']);
            
        } catch (Exception $e) {  
            $this->error('同步失败：' . $e->getMessage());
            $errorParams = ['file' => $e->getFile(), 'line' => $e->getLine(), 'error' => $e->getMessage(), 'trace' => $e->getTraceAsString()];
            AppLog::logError('佣金单列表同步失败', $errorParams, 'sync_order_list'); 
            EmailQueue::rPush('佣金单列表同步失败', json_encode($errorParams), ['song.shen@kuxuan-inc.com'], '微店推客带货');
        }
    }

    private function shouldExit(): bool
    {
        if (time() - $this->startTime >= self::MAX_EXECUTION_TIME) {
            $this->warn('脚本已运行10分钟，自动退出');
            return true;
        }
        return false;
    }
} 