<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Model;

class Order extends Model
{
    protected $table = 'wechat_order'; // 表名
    protected $primaryKey = 'id'; // 主键 
    public $timestamps = false; // 不自动维护时间戳
    protected static $unguarded = true;
    
    const CREATED_AT = 'create_time'; // 创建时间
    const UPDATED_AT = 'update_time'; // 更新时间 

    const ORDER_INFO_QUEUE = 'wechatStore:order_info_queue'; // 订单详情队列
    const ORDER_INFO_FAILED_QUEUE = 'wechatStore:order_info_failed_queue'; // 订单详情失败队列  

    // 佣金状态常量
    const COMMISSION_STATUS_PENDING = 20;    // 未结算
    const COMMISSION_STATUS_SETTLED = 100;   // 已结算
    const COMMISSION_STATUS_CANCELED = 200;  // 取消结算

    // 订单状态常量
    const ORDER_STATUS_PENDING_PAYMENT = 10;     // 待付款
    const ORDER_STATUS_PENDING_DELIVERY = 20;    // 待发货
    const ORDER_STATUS_PARTIAL_DELIVERY = 21;    // 部分发货
    const ORDER_STATUS_PENDING_RECEIPT = 30;     // 待收货
    const ORDER_STATUS_COMPLETED = 100;          // 完成
    const ORDER_STATUS_AFTER_SALE_CANCEL = 200;  // 售后取消
    const ORDER_STATUS_CANCEL = 250;             // 取消订单

    /**
     * 关联商品SKU
     */
    public function sku()
    {
        return $this->belongsTo(GoodsSkus::class, 'sku_id', 'sku_id');
    }

    /**
     * 关联商品信息
     */
    public function goods()
    {
        return $this->belongsTo(Goods::class, 'product_id', 'product_id');
    }

    /**
     * 检查订单是否已完成
     */
    public function isCompleted()
    {
        return $this->order_status === self::ORDER_STATUS_COMPLETED;
    }

    /**
     * 检查佣金是否已结算
     */
    public function isCommissionSettled()
    {
        return $this->commission_status === self::COMMISSION_STATUS_SETTLED;
    }

    /**
     * 获取订单状态文本
     */
    public function getOrderStatusText()
    {
        $statusMap = [
            self::ORDER_STATUS_PENDING_PAYMENT => '待付款',
            self::ORDER_STATUS_PENDING_DELIVERY => '待发货',
            self::ORDER_STATUS_PARTIAL_DELIVERY => '部分发货',
            self::ORDER_STATUS_PENDING_RECEIPT => '待收货',
            self::ORDER_STATUS_COMPLETED => '已完成',
            self::ORDER_STATUS_AFTER_SALE_CANCEL => '售后取消',
            self::ORDER_STATUS_CANCEL => '已取消'
        ];

        return $statusMap[$this->order_status] ?? '未知状态';
    }

    /**
     * 获取佣金状态文本
     */
    public function getCommissionStatusText()
    {
        $statusMap = [
            self::COMMISSION_STATUS_PENDING => '未结算',
            self::COMMISSION_STATUS_SETTLED => '已结算',
            self::COMMISSION_STATUS_CANCELED => '取消结算'
        ];

        return $statusMap[$this->commission_status] ?? '未知状态';
    }

    /**
     * 计算总佣金金额
     */
    public function getTotalCommission()
    {
        return $this->finder_amount + $this->service_amount;
    }

    public static function saveOrderDetail($detail) {
        $orderInfo = $detail['commssion_order'];
        return Order::updateOrCreate(
            [
                'shop_appid' => $orderInfo['order_detail']['shop_info']['appid'],
                'order_id' => $orderInfo['order_id']
            ],
            [
                'sku_id' => $orderInfo['sku_id'] ?? '',
                'order_create_time' => isset($orderInfo['create_time']) ? date('Y-m-d H:i:s', $orderInfo['create_time']) : null,
                'order_update_time' => isset($orderInfo['update_time']) ? date('Y-m-d H:i:s', $orderInfo['update_time']) : null,
                'commission_status' => $orderInfo['status'] ?? 0,
                'product_id' => $orderInfo['order_detail']['product_info']['product_id'],
                'thumb_img' => $orderInfo['order_detail']['product_info']['thumb_img'] ?? '',
                'actual_payment' => $orderInfo['order_detail']['product_info']['actual_payment'] ?? 0,
                'product_title' => $orderInfo['order_detail']['product_info']['title'] ?? '',
                'order_status' => $orderInfo['order_detail']['order_info']['order_status'] ?? 0,
                'finder_nickname' => $orderInfo['order_detail']['commission_info']['finder_info']['nickname'] ?? '',
                'finder_ratio' => $orderInfo['order_detail']['commission_info']['finder_info']['ratio'] ?? 0,
                'finder_amount' => $orderInfo['order_detail']['commission_info']['finder_info']['amount'] ?? 0,
                'openfinderid' => $orderInfo['order_detail']['commission_info']['finder_info']['openfinderid'] ?? '',
                'service_ratio' => $orderInfo['order_detail']['commission_info']['service_info']['ratio'] ?? 0,
                'service_amount' => $orderInfo['order_detail']['commission_info']['service_info']['amount'] ?? 0,
                'profit_sharding_suc_time' => isset($orderInfo['profit_sharding_suc_time']) ? date('Y-m-d H:i:s', $orderInfo['profit_sharding_suc_time']) : null,
                'enable' => 1
            ] 
        );
    }

    /**
     * 获取订单列表数据
     *
     * @param int $userId 用户ID
     * @param int $page 页码
     * @param int $pageSize 每页数量
     * @param int|null $commissionStatus 佣金状态
     * @param int|null $orderStatus 订单状态
     * @return array [$data, $count]
     */
    public static function getOrderListData($userId, $page = 1, $pageSize = 20, $commissionStatus = null, $orderStatus = null)
    {
        $query = self::query()
            ->where('user_id', $userId)
            ->orderBy('create_time', 'desc');

        if ($commissionStatus !== null) {
            $query->where('commission_status', $commissionStatus);
        }

        if ($orderStatus !== null) {
            $query->where('order_status', $orderStatus);
        }

        $count = $query->count();
        $offset = ($page - 1) * $pageSize;

        $data = $query->offset($offset)
            ->limit($pageSize)
            ->get();

        // 数据格式化
        foreach ($data as $item) {
            $item->thumb_img = $item->thumb_img ?: '';
            $item->commission_amount = round($item->commission_amount / 100, 2);
            $item->order_amount = round($item->order_amount / 100, 2);
            $item->create_time = date('Y-m-d H:i:s', strtotime($item->create_time));
            $item->update_time = date('Y-m-d H:i:s', strtotime($item->update_time));
        }

        return [$data, $count];
    }

    // 获取用户预估佣金
    public static function getUserEstimateCommission($userId, $startTime, $endTime) {
        return self::where('user_id', $userId)
            ->whereIn('commission_status', [self::COMMISSION_STATUS_SETTLED, self::COMMISSION_STATUS_PENDING])// 已结算和未结算
            ->whereNotIn('order_status', [self::ORDER_STATUS_CANCEL, self::ORDER_STATUS_AFTER_SALE_CANCEL])// 已取消和售后取消
            ->where('order_create_time', '>=', $startTime)
            ->where('order_create_time', '<=', $endTime)
            ->sum('finder_amount');
    }

    // 获取用户结算佣金
    public static function getSettledCommission($userId, $startTime, $endTime) {
        return self::where('user_id', $userId)
            ->where('commission_status', self::COMMISSION_STATUS_SETTLED)
            ->whereNotIn('order_status', [self::ORDER_STATUS_CANCEL, self::ORDER_STATUS_AFTER_SALE_CANCEL])// 已取消和售后取消
            ->where('profit_sharding_suc_time', '>=', $startTime)
            ->where('profit_sharding_suc_time', '<=', $endTime)
            ->sum('finder_amount');
    }

    // 累计结算佣金
    public static function getTotalSettledCommission($userId) {
        return self::where('user_id', $userId)
            ->where('commission_status', self::COMMISSION_STATUS_SETTLED)// 已结算
            ->whereNotIn('order_status', [self::ORDER_STATUS_CANCEL, self::ORDER_STATUS_AFTER_SALE_CANCEL])// 已取消和售后取消
            ->sum('finder_amount');
    } 
    
    public static function getWaitSettledCommission($userId) {
        return self::where('user_id', $userId)
            ->where('commission_status', self::COMMISSION_STATUS_PENDING)// 未结算
            ->whereNotIn('order_status', [self::ORDER_STATUS_CANCEL, self::ORDER_STATUS_AFTER_SALE_CANCEL])// 已取消和售后取消
            ->sum('finder_amount');
    }

    public static function moneyReturn($level, $money){
        # 加上基数
        $money = 0.90*$money;
        // if( $share == 1){  
        //     return $money*0.6;   
        // }
        if($level==3 || $level==4){
            $tmp_price = $money*0.9;
        }else if($level==2 || $level==1){
            $tmp_price = $money*0.6;
        }else{
            $tmp_price = 0;
        }

        return $tmp_price;
    }
}