<?php

namespace App\Service;

use App\Log as AppLog;
use App\Models\Goods;
use App\Models\GoodsSkus;
use App\Models\Order;
use App\Support\EmailQueue;
use App\Support\weChatApi;
use Exception;

class GoodsService
{
    /**
     * 获取商品列表
     */
    public static function getGoodsList(array $params)
    {
        try {
            list($data, $total) = Goods::getGoodsListData($params);

            $userLevel = UserService::returnUserLevel($params['user_id']);

            # 数据格式化
            foreach ($data as $item) {
                $item->head_img_list = json_decode($item->head_imgs, true);

                $item->sale_price = round($item->min_sale_price / 100, 2);
                $item->stock_num = $item->max_stock_num;

                // 根据用户查询分佣比例以及佣金金额（参考猎豆优选）
                $commissionPrice = $item->sale_price * $item->normal_commission_ratio / 100;
                $item->commission_amount = Order::moneyReturn($userLevel, $commissionPrice);
            }

            return [$data, $total]; 
        } catch (Exception $e) {
            AppLog::logError('获取商品列表失败', ['params' => $params, 'file' => $e->getFile(), 'line' => $e->getLine(), 'message' => $e->getMessage(), 'trace' => $e->getTraceAsString()], 'goodsListApi');
            EmailQueue::rPush('获取商品列表失败', json_encode(['params' => $params, 'file' => $e->getFile(), 'line' => $e->getLine(), 'message' => $e->getMessage(), 'trace' => $e->getTraceAsString()], 256), ['song.shen@kuxuan-inc.com'], '微店推客带货');
            return [[], 0];
        }
    }

    /**
     * 获取商品详情
     */
    public static function getGoodsDetail($productId, $sharerOpenId)
    {
        try {
            // 获取商品基本信息和SKU
            $goods = Goods::getGoodsDetail($productId);

            // 构建商品详情数据
            $detail = [
                'product_id' => $goods->product_id,
                'title' => $goods->title,
                'sub_title' => $goods->sub_title,
                'images' => [
                    'head' => json_decode($goods->head_imgs, 1),
                    'desc' => json_decode($goods->desc_imgs, 1)
                ],
                'description' => $goods->desc_text,
                'commission_amount' => 0,
                'skus' => []
            ];

            // 根据推客获取推广佣金金额

            //todo 获取商品推广链接
            if ($sharerOpenId) {
                $shareResponse= weChatApi::getPromoteLink(null, $sharerOpenId, $productId, null, null);
            } else {
                $shareResponse = weChatApi::getPromoteLink($goods->appid, null, $productId, null, null);
            }

            if (isset($shareResponse['errcode']) && $shareResponse['errcode'] == -1) {
                AppLog::logError('获取商品推广链接失败', [
                    'product_id' => $productId,
                    'sharer_open_id' => $sharerOpenId,
                    'errcode' => $shareResponse['errcode'],
                    'errmsg' => $shareResponse['errmsg'],
                ], 'goodsDetailApi');
                EmailQueue::rPush('获取商品推广链接失败', json_encode([
                    'product_id' => $productId,
                    'sharer_open_id' => $sharerOpenId,
                    'errcode' => $shareResponse['errcode'],
                    'errmsg' => $shareResponse['errmsg'],
                ], 256), ['song.shen@kuxuan-inc.com'], '微店推客带货');
            }
            $detail['share_link'] = $shareResponse['short_link'] ?? '';

            // 处理SKU信息
            if ($goods->skus->isNotEmpty()) {
                foreach ($goods->skus as $sku) {
                    $detail['skus'][] = [
                        'sku_id' => $sku->sku_id,
                        'price' => round($sku->sale_price / 100, 2),
                        'stock' => $sku->stock_num,
                        'thumb' => $sku->thumb_img,
                        'attrs' => json_decode($sku->attrs, true) ?? [],
                    ];
                }

                // 设置商品价格区间
                $prices = $goods->skus->pluck('sale_price')->sort();
                $detail['price_range'] = [
                    'min' => round($prices->first() / 100, 2),
                    'max' => round($prices->last() / 100, 2),
                ];
            }

            AppLog::logInfo('获取商品详情成功', [
                'product_id' => $productId,
                'sharer_open_id' => $sharerOpenId
            ]);

            return $detail;

        } catch (Exception $e) {
            AppLog::logError('商品不存在', [
                'product_id' => $productId,
                'sharer_open_id' => $sharerOpenId,
                'file' => $e->getFile(),
                'line' => $e->getLine(),
                'message' => $e->getMessage(),
                'trace' => $e->getTraceAsString(),
            ], 'goodsDetailApi');
            EmailQueue::rPush('商品不存在', json_encode([
                'product_id' => $productId,
                'sharer_open_id' => $sharerOpenId,
                'file' => $e->getFile(),
                'line' => $e->getLine(),
                'message' => $e->getMessage(),
                'trace' => $e->getTraceAsString(),
            ], 256), ['song.shen@kuxuan-inc.com'], '微店推客带货');
            return [];
        }
    }
} 