<?php

namespace App\Service;

use App\Log as AppLog;
use App\Models\AccountDetail;
use App\Models\Order;
use App\RedisModel;
use App\Support\EmailQueue;
use App\Support\weChatApi;
use Exception;

class OrderService
{
    const QUEUE_KEY = 'order_detail_queue';

    /**
     * 处理订单列表数据
     */
    public static function processOrderList($appId, $startTime, $endTime, $nextKey = '')
    {
        $finderId = null;
        $openFinderId = null;
        $createTimeRange = null;
        $updateTimeRange = [
          'start_time' => $startTime,
          'end_time' => $endTime
        ];
        $orderId = null;
        $pageSize = 30;  

        try {
            // 调用微信API获取佣金单列表
            $response = weChatApi::getOrderList($appId, $finderId, $openFinderId, $createTimeRange, $updateTimeRange, $orderId, $pageSize, $nextKey);

            if (!$response || $response['errcode'] !== 0 || empty($response['list'])) {
                return [
                    'success' => false,
                    'next_key' => '',
                    'count' => 0,
                    'has_more' => false
                ];
            }

            // 将订单ID存入Redis队列
            foreach ($response['list'] as $order) {
                RedisModel::rpush(self::QUEUE_KEY, json_encode([
                    'sku_id' => $order['sku_id'],
                    'order_id' => $order['order_id']
                ]));
            }

            return [
                'success' => true,
                'next_key' => $response['next_key'] ?? '',
                'count' => count($response['list']),
                'has_more' => $response['has_more'] ?? false  
            ];

        } catch (Exception $e) {
            self::handleError('获取佣金单列表失败', $e);
            $errorParams = ['appId' => $appId, 'startTime' => $startTime, 'endTime' => $endTime, 'nextKey' => $nextKey, 'file' => $e->getFile(), 'line' => $e->getLine(), 'error' => $e->getMessage(), 'trace' => $e->getTraceAsString()];
            AppLog::logError('获取佣金单列表失败', $errorParams, 'sync_order_list'); 
            EmailQueue::rPush('获取佣金单列表失败', json_encode($errorParams), ['song.shen@kuxuan-inc.com'], '微店推客带货'); 
            return [
              'success' => false,
              'next_key' => '',
              'count' => 0,
              'has_more' => false
          ];
        }
    }

    /**
     * 处理订单详情数据
     */
    public static function processOrderDetail($orderId, $skuId)
    {
        try {
            // 获取佣金单详情
            $detail = weChatApi::getOrderInfo(
                $orderId, $skuId
            );

            if (!$detail || $detail['errcode'] !== 0) {
                return false;
            }

            return $detail;

        } catch (Exception $e) {
            self::handleError("处理佣金单 {$orderId} 详情失败", $e);
            return false;
        }
    }

    /**
     * 获取队列长度
     */
    public static function getQueueLength($queueKey = Order::ORDER_INFO_QUEUE)
    {
        return RedisModel::llen($queueKey);
    }

    /**
     * 处理错误
     */
    public static function handleError($message, Exception $e)
    {
        $errParams = [
            'message' => $message,
            'file' => $e->getFile(),
            'line' => $e->getLine(),
            'error' => $e->getMessage(), 
            'trace' => $e->getTraceAsString()
        ];
        AppLog::logError('同步失败', $errParams, 'sync_order_list');
        EmailQueue::rPush('同步失败', json_encode($errParams), ['song.shen@kuxuan-inc.com'], '微店推客带货'); 
    }

    /**
     * 获取订单列表
     *
     * @param int $userId 用户ID
     * @param int $page 页码
     * @param int $pageSize 每页数量
     * @param int|null $commissionStatus 佣金状态
     * @param int|null $orderStatus 订单状态
     * @return array [$data, $count]
     */
    public static function getOrderList($userId, $page = 1, $pageSize = 20, $commissionStatus = null, $orderStatus = null)
    {
        try {
            return Order::getOrderListData($userId, $page, $pageSize, $commissionStatus, $orderStatus);
        } catch (Exception $e) {
            AppLog::logError('获取订单列表查询失败', [
                'user_id' => $userId,
                'page' => $page,
                'page_size' => $pageSize,
                'commission_status' => $commissionStatus,
                'order_status' => $orderStatus,
                'file' => $e->getFile(),
                'line' => $e->getLine(),
                'message' => $e->getMessage(),
                'trace' => $e->getTraceAsString(),
            ], 'orderListApi');

            return [[], 0];
        }
    }

    // 用户收益统计
    public static function estimatesOfIncome($userId) {
        $thisMonthBeginTime=date("Y-m-d H:i:s",mktime(0, 0 , 0,date("m"),1,date("Y")));
        $thisDayBeginTime = date("Y-m-d H:i:s",mktime(0, 0 , 0));//当天开始时间
        $thisDayEndTime = date("Y-m-d H:i:s");//当天结束时间

        // 本月预估收入
        $thisIncomePredict=Order::getUserEstimateCommission($userId, $thisMonthBeginTime, $thisDayEndTime);
        // 今日预估收入
        $thisDayIncomePredict=Order::getUserEstimateCommission($userId, $thisDayBeginTime, $thisDayEndTime);

        // 今日结算收入
        $thisDaySettledCommission=Order::getSettledCommission($userId, $thisDayBeginTime, $thisDayEndTime);
        // 本月结算收入
        $thisMonthSettledCommission=Order::getSettledCommission($userId, $thisMonthBeginTime, $thisDayEndTime);

        // 累计结算收入
        $totalSettledCommission=Order::getTotalSettledCommission($userId);
        // 待结算收入
        $waitSettledCommission=Order::getWaitSettledCommission($userId);

        $data['thisIncomePredict']=self::changeBranchToElement($thisIncomePredict);//本月预估
        $data['thisMonthSettledCommission']=self::changeBranchToElement($thisMonthSettledCommission);//本月到账
        $data['thisDayIncomePredict']=self::changeBranchToElement($thisDayIncomePredict);//今日预估
        $data['thisDaySettledCommission']=self::changeBranchToElement($thisDaySettledCommission);//今日到账
        $data['totalSettledCommission']=self::changeBranchToElement($totalSettledCommission);//累计结算收入
        $data['waitSettledCommission']=self::changeBranchToElement($waitSettledCommission);//待结算收入

        return $data;
    }

    public static function changeGoodsBranchToElement($num) {
        return Number_format($num,2, '.','');
    } 

    public static function changeBranchToElement($num){
        return Number_format(intval($num)/100,2, '.','');
    }

    // 用户账户余额
    public static function accountPrice($userId, $type) {
        if($type==2){
            $price=AccountDetail::getIncomeByUserId($userId);
        }else if($type==0){
            $price=AccountDetail::getWithdrawalByUserId($userId);
        }else {
            $price=AccountDetail::getWithdrawalByUserIdApproved($userId);
        }
        $price=OrderService::changeBranchToElement($price);
        return $price;
    }

    // 用户收益明细
    public static function myEarningsDetail($userId) {
        $data = AccountDetail::getMyEarningsDetail($userId);
        return $data;
    }
}